/*****************************************************************************
* luaEngine Lua Engine for Qt
* Copyright (C) 2018 Syping
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*****************************************************************************/

#include "LuaEngineStyle.h"
#include "LuaEngineGui.h"
#include <QApplication>
#include <QFont>
#include <QFile>

#ifdef _WIN32
#include "windows.h"
#endif

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    QApplication::setStyle(new LuaEngineStyle());

#ifdef _WIN32
#if QT_VERSION >= 0x050400
    if (QSysInfo::windowsVersion() >= 0x0080)
    {
        NONCLIENTMETRICS ncm;
        ncm.cbSize = sizeof(ncm);
        SystemParametersInfo(SPI_GETNONCLIENTMETRICS, ncm.cbSize, &ncm, 0);
        LOGFONTW uiFont = ncm.lfMessageFont;
        QString uiFontStr(QString::fromStdWString(std::wstring(uiFont.lfFaceName)));
        QFont appFont(uiFontStr, 9);
        a.setFont(appFont);
    }
#endif
#endif

    QFile *luaEngineStyleFile = new QFile(":/luaenginestyle/luaenginestyle.qss");
    if (luaEngineStyleFile->open(QFile::ReadOnly))
    {
        a.setStyleSheet(QString::fromUtf8(luaEngineStyleFile->readAll()));
    }
    delete luaEngineStyleFile;

    QFile luaScript(":/lua/app.lua");

    LuaEngineGui luaEngineGui;
    luaEngineGui.executeLuaScript(&luaScript);

    QVariantList arguments;
    for (const QString argument : a.arguments()) {
        arguments << QVariant::fromValue(argument);
    }

    if (luaEngineGui.executeLuaFunction("main", arguments, true)) {
        QVariant variant = luaEngineGui.returnVariant();
        if (variant.type() == QVariant::Int) {
            return variant.toInt();
        }
        else if (variant.type() == QVariant::String) {
            if (variant.toString() == "GuiExecuted") {
                return a.exec();
            }
            else {
                return 1;
            }
        }
        else {
            return 1;
        }
    }
    else {
        return 1;
    }

    return a.exec();
}
